@@ -32,7 +32,6 @@
 #include <time.h>
 
 #include <curl/curl.h>
-#include <curl/mprintf.h>
 
 #if defined(_MSC_VER) && (_MSC_VER < 1900)
 #define snprintf _snprintf
@@ -48,7 +47,7 @@ static int print_cookies(CURL *curl)
   printf("Cookies, curl knows:\n");
   res = curl_easy_getinfo(curl, CURLINFO_COOKIELIST, &cookies);
   if(res != CURLE_OK) {
-    fprintf(stderr, "Curl curl_easy_getinfo failed: %s\n",
+    fprintf(stderr, "curl curl_easy_getinfo failed: %s\n",
             curl_easy_strerror(res));
     return 1;
   }
@@ -73,7 +72,10 @@ main(void)
   CURL *curl;
   CURLcode res;
 
-  curl_global_init(CURL_GLOBAL_ALL);
+  res = curl_global_init(CURL_GLOBAL_ALL);
+  if(res)
+    return (int)res;
+
   curl = curl_easy_init();
   if(curl) {
     char nline[512];
@@ -83,7 +85,7 @@ main(void)
     curl_easy_setopt(curl, CURLOPT_COOKIEFILE, ""); /* start cookie engine */
     res = curl_easy_perform(curl);
     if(res != CURLE_OK) {
-      fprintf(stderr, "Curl perform failed: %s\n", curl_easy_strerror(res));
+      fprintf(stderr, "curl perform failed: %s\n", curl_easy_strerror(res));
       return 1;
     }
 
@@ -103,7 +105,7 @@ main(void)
              "PREF", "hello example, I like you!");
     res = curl_easy_setopt(curl, CURLOPT_COOKIELIST, nline);
     if(res != CURLE_OK) {
-      fprintf(stderr, "Curl curl_easy_setopt failed: %s\n",
+      fprintf(stderr, "curl curl_easy_setopt failed: %s\n",
               curl_easy_strerror(res));
       return 1;
     }
@@ -118,7 +120,7 @@ main(void)
       "expires=Sun, 17-Jan-2038 19:14:07 GMT; path=/; domain=.example.com");
     res = curl_easy_setopt(curl, CURLOPT_COOKIELIST, nline);
     if(res != CURLE_OK) {
-      fprintf(stderr, "Curl curl_easy_setopt failed: %s\n",
+      fprintf(stderr, "curl curl_easy_setopt failed: %s\n",
               curl_easy_strerror(res));
       return 1;
     }
@@ -127,14 +129,14 @@ main(void)
 
     res = curl_easy_perform(curl);
     if(res != CURLE_OK) {
-      fprintf(stderr, "Curl perform failed: %s\n", curl_easy_strerror(res));
+      fprintf(stderr, "curl perform failed: %s\n", curl_easy_strerror(res));
       return 1;
     }
 
     curl_easy_cleanup(curl);
   }
   else {
-    fprintf(stderr, "Curl init failed!\n");
+    fprintf(stderr, "curl init failed!\n");
     return 1;
   }
 
